/*
 * Xailer source code:
 *
 * ToolTip.prg
 * Clases TToolTip() y TBalloon()
 *
 * Copyright 2003, 2007 Jose F. Gimenez
 * Copyright 2003, 2007 Xailer.com
 * All rights reserved
 *
 */

#include "Xailer.ch"
#include "CommCtrl.api"

//--------------------------------------------------------------------------

CLASS XBalloon FROM TToolTip

PROTECTED:
   DATA nCtlStyle INIT nOr( TTS_BALLOON, TTS_NOPREFIX )

ENDCLASS

//--------------------------------------------------------------------------

CLASS XToolTip FROM TWinObject

PROTECTED:
   DATA nCtlStyle INIT TTS_NOPREFIX

PUBLIC:
   PROPERTY cText
   PROPERTY cTitle
   PROPERTY nIcon
   PROPERTY lAlwaysTip     INIT .F. WRITE METHOD SetAlwaysTip
   PROPERTY lCloseButton   INIT .F. WRITE METHOD SetCloseButton

   EVENT OnShow( oSender ) // --> Nil
   EVENT OnHide( oSender ) // --> Nil
   EVENT OnLinkClick( oSender ) // --> Nil

   METHOD Create() CONSTRUCTOR
   METHOD Destroy( lFree )

   METHOD SetAlwaysTip( lAlwaysTip ) // --> lResult
   METHOD SetCloseButton( lCloseButton ) // --> lResult
   METHOD AddTool( oControl, aZone, nId ) // --> Nil
   METHOD SetToolInfo( oControl ) // --> Nil
   METHOD DelTool( oControl, nId ) // --> Nil

   METHOD Show() // --> Nil
   METHOD Hide()     INLINE PostMessage( ::Handle, TTM_POP ) // --> Nil

RESERVED:
   DATA oForm        AS TForm

   METHOD Link()     EXTERN WndLink()
   METHOD Unlink()   EXTERN WndUnlink()
   METHOD Notify()
   METHOD WMShowWindow( nWParam, nLParam )   INLINE IIF( nWParam == 0, ::OnHide(), ::OnShow() ), Nil
   METHOD WMLButtonUp( nWParam, nLParam )    INLINE IIF( IsAppThemed(),, ::Hide() ) , Nil

ENDCLASS

//--------------------------------------------------------------------------

METHOD Create( oParent ) CLASS XToolTip

   InitCommonControls( ICC_WIN95_CLASSES )

   UPDATE ::oParent TO oParent

   IF ( ::Handle := CreateWindow( TOOLTIPS_CLASS, "", ::nCtlStyle ) ) != 0
      ::Link()
      SendMessage( ::Handle, TTM_SETMAXTIPWIDTH, 0, 32768 )
      SetWindowPos( ::Handle, HWND_TOPMOST, 0, 0, 0, 0, nOr( SWP_NOMOVE, SWP_NOSIZE, SWP_NOACTIVATE ) )
   ENDIF
   ::SetAlwaysTip( ::lAlwaysTip )

RETURN Self

//--------------------------------------------------------------------------

METHOD Destroy( lFree ) CLASS XToolTip

   ::UnLink()
   DestroyWindow( ::Handle )

RETURN ::Super:Destroy( lFree )

//--------------------------------------------------------------------------

METHOD SetAlwaysTip( lAlwaysTip ) CLASS XToolTip

   UPDATE ::FlAlwaysTip TO lAlwaysTip

   IF !Empty( ::Handle )
      IF ::FlAlwaysTip
         SetWindowLong( ::Handle, GWL_STYLE, nOr( GetWindowLong( ::Handle, GWL_STYLE ), TTS_ALWAYSTIP ) )
      ELSE
         SetWindowLong( ::Handle, GWL_STYLE, nExclude( GetWindowLong( ::Handle, GWL_STYLE ), TTS_ALWAYSTIP ) )
      ENDIF
   ENDIF

RETURN ::FlAlwaysTip

//--------------------------------------------------------------------------

METHOD SetCloseButton( lCloseButton ) CLASS XToolTip

   UPDATE ::FlCloseButton TO lCloseButton

   IF ::FlCloseButton
      ::nCtlStyle := nOr( ::nCtlStyle, TTS_CLOSE )
   ELSE
      ::nCtlStyle := nExclude( ::nCtlStyle, TTS_CLOSE )
   ENDIF

RETURN ::FlCloseButton

//--------------------------------------------------------------------------

#pragma BEGINDUMP

#define _WIN32_IE 0x0500

#include <windows.h>
#include <commctrl.h>
#include <xailer.h>

#define TTN_LINKCLICK   (TTN_FIRST - 3)

//--------------------------------------------------------------------------

static void XTooltip_ToolInfo( UINT nMsg )
{
   PHB_ITEM Self = hb_stackSelfItem();
   HANDLE handle = GetHandleOf( Self );
   PHB_ITEM oControl = hb_param( 1, HB_IT_OBJECT );
   TOOLINFO ti;

   if ( oControl )
      {
      ti.cbSize   = sizeof( TOOLINFO );
      XA_ObjSend( oControl, hb_clsIsParent( XA_ObjGetNL( Self, "ClassH" ), "XBALLOON" ) ? "cBalloon" : "cTooltip" );
      if ( HB_ISCHAR( -1 ) )
         ti.lpszText = LPSTR_TEXTCALLBACK; //hb_parc( -1 );
      else
         {
         ti.lpszText = (LPVOID) hb_parnl( -1 );
         ti.hinst = GetInstance( 0 );
         }
      if( HB_ISNUM( 3 ) )
         {
         PHB_ITEM pRect = hb_param( 2, HB_IT_ARRAY );
         ti.uFlags      = TTF_SUBCLASS;
         ti.hwnd        = (HWND) GetHandleOf( oControl );
         ti.uId         = hb_parnl( 3 );
         ti.rect.left   = hb_arrayGetNL( pRect, 1 );
         ti.rect.top    = hb_arrayGetNL( pRect, 2 );
         ti.rect.right  = hb_arrayGetNL( pRect, 3 );
         ti.rect.bottom = hb_arrayGetNL( pRect, 4 );
         }
      else
         {
         ti.uFlags = TTF_IDISHWND | TTF_SUBCLASS | TTF_TRANSPARENT;
         ti.hwnd   = GetHandleOf( XA_ObjGetItem( oControl, "GetForm" ) );
         ti.uId    = (UINT_PTR) GetHandleOf( oControl );
         }
      ti.lParam   = ti.uId;
      SendMessage( handle, nMsg, 0, (LPARAM) (LPTOOLINFO) &ti );

      if( XA_ObjGetLen( Self, "cTitle" ) )
         SendMessage( handle, TTM_SETTITLE, XA_ObjGetNL( Self, "nIcon" ), (LPARAM) XA_ObjGetC( Self, "cTitle" ) );
      else if( XA_ObjGetLen( oControl, "cTooltipTitle" ) )
         SendMessage( handle, TTM_SETTITLE, XA_ObjGetNL( oControl, "nTooltipIcon" ), (LPARAM) XA_ObjGetC( oControl, "cTooltipTitle" ) );
      }
   hb_ret();
}

//--------------------------------------------------------------------------

HB_FUNC_STATIC( XTOOLTIP_ADDTOOL )
{
   XTooltip_ToolInfo( TTM_ADDTOOL );
}

//--------------------------------------------------------------------------

HB_FUNC_STATIC( XTOOLTIP_SETTOOLINFO )
{
   XTooltip_ToolInfo( TTM_SETTOOLINFO );
}

//--------------------------------------------------------------------------

HB_FUNC_STATIC( XTOOLTIP_DELTOOL )
{
   PHB_ITEM Self = hb_stackSelfItem();
   HANDLE handle = GetHandleOf( Self );
   PHB_ITEM oControl = hb_param( 1, HB_IT_OBJECT );
   TOOLINFO ti;

   if ( oControl )
      {
      ti.cbSize = sizeof( TOOLINFO );
      if( HB_ISNUM( 2 ) )
         {
         ti.hwnd = (HWND) GetHandleOf( oControl );
         ti.uId  = hb_parnl( 2 );
         }
      else
         {
         ti.hwnd = GetHandleOf( XA_ObjGetItem( oControl, "GetForm" ) );
         ti.uId  = (UINT_PTR) GetHandleOf( oControl );
         }
      SendMessage( handle, TTM_DELTOOL, 0, (LPARAM) (LPTOOLINFO) &ti );
      }
   hb_ret();
}

//--------------------------------------------------------------------------

HB_FUNC_STATIC( XTOOLTIP_SHOW )
{
   PHB_ITEM Self = hb_stackSelfItem();
   HANDLE handle = GetHandleOf( Self );
   PHB_ITEM oControl;
   TOOLINFO ti;
   RECT rect;

   if( HB_ISOBJECT( 3 ) )
      oControl = hb_itemNew( hb_param( 3, HB_IT_OBJECT ) );
   else
      oControl = XA_ObjGetItemCopy( Self, "oParent" );

   if( oControl && !HB_IS_NIL( oControl ) )
      {
      ti.cbSize   = sizeof( TOOLINFO );
      ti.lpszText = XA_ObjGetC( Self, "cText" );
      if( ( ti.lpszText == NULL ) || ( *ti.lpszText == 0 ) )
         ti.lpszText = XA_ObjGetC( oControl, hb_clsIsParent( XA_ObjGetNL( Self, "ClassH" ), "XBALLOON" ) ? "cBalloon" : "cTooltip" );
      ti.uFlags   = TTF_IDISHWND | TTF_TRACK;
      ti.hwnd     = GetHandleOf( XA_ObjGetItem( oControl, "GetForm" ) );
      ti.uId      = (UINT_PTR) GetHandleOf( oControl );
      ti.lParam   = ti.uId;

      if( XA_ObjGetLen( Self, "cTitle" ) )
         SendMessage( handle, TTM_SETTITLE, XA_ObjGetNL( Self, "nIcon" ), (LPARAM) XA_ObjGetC( Self, "cTitle" ) );
      else if( XA_ObjGetLen( oControl, "cTooltipTitle" ) )
         SendMessage( handle, TTM_SETTITLE, XA_ObjGetNL( oControl, "nTooltipIcon" ), (LPARAM) XA_ObjGetC( oControl, "cTooltipTitle" ) );

      if( HB_ISNIL( 1 ) || HB_ISNIL( 2 ) )
         GetWindowRect( (HWND) ti.uId, &rect );
      else
         {
         rect.left = hb_parnl( 1 );
         rect.top = hb_parnl( 2 );
         ClientToScreen( (HWND) ti.uId, (LPPOINT) &rect );
         }
      SendMessage( handle, TTM_ADDTOOL, 0, (LPARAM) (LPTOOLINFO) &ti );
      SendMessage( handle, TTM_TRACKPOSITION, 0, MAKELONG( rect.left, rect.top ) );
      SendMessage( handle, TTM_TRACKACTIVATE, TRUE, (LPARAM) &ti );
      }
   hb_itemRelease( oControl);
}

//--------------------------------------------------------------------------

HB_FUNC_STATIC( XTOOLTIP_NOTIFY )
{
   PHB_ITEM Self = hb_stackSelfItem();
   LPNMHDR nmhdr = (LPNMHDR) hb_parnl( 2 );

   if( nmhdr->code )
      {
      switch( nmhdr->code )
         {
         case TTN_SHOW:
            XA_ObjSend( Self, "OnShow" );
            break;
         case TTN_POP:
            XA_ObjSend( Self, "OnHide" );
            if( hb_parnl( 1 ) && IsWindow( (HWND) nmhdr->idFrom ) )
               SendMessage( (HWND) nmhdr->idFrom, WM_NOTIFY, 0, (LPARAM) nmhdr );
            break;
         case TTN_LINKCLICK:
            XA_ObjSend( Self, "OnLinkClick" );
         }
      }
   hb_ret();
}

//--------------------------------------------------------------------------

#pragma ENDDUMP
